@@ -2,13 +2,22 @@ import hashlib
 import pickle
 import re
 import socket
-from .compat import urlencode, Request, urlopen, URLError, HTTPError, get_charset
+from .compat import get_charset
+from .compat import HTTPError
+from .compat import Request
+from .compat import urlencode
+from .compat import URLError
+from .compat import urlopen
 try:
     import simplejson as json
+    InvalidJson = json.JSONDecodeError
 except ImportError:
     import json
+    InvalidJson = ValueError
 
-from micawber.exceptions import ProviderException, ProviderNotFoundException
+from micawber.exceptions import InvalidResponseException
+from micawber.exceptions import ProviderException
+from micawber.exceptions import ProviderNotFoundException
 
 
 class Provider(object):
@@ -24,23 +33,14 @@ class Provider(object):
         socket.setdefaulttimeout(self.socket_timeout)
         req = Request(url, headers={'User-Agent': self.user_agent})
         try:
-            resp = urlopen(req)
+            resp = fetch(req)
         except URLError:
             return False
         except HTTPError:
             return False
         except socket.timeout:
             return False
-
-        if resp.code < 200 or resp.code >= 300:
-            return False
-
-        # by RFC, default HTTP charset is ISO-8859-1
-        charset = get_charset(resp) or 'iso-8859-1'
-
-        content = resp.read().decode(charset)
-        resp.close()
-        return content
+        return resp
 
     def encode_params(self, url, **extra_params):
         params = dict(self.base_params)
@@ -64,9 +64,16 @@ class Provider(object):
             raise ProviderException('Error fetching "%s"' % endpoint_url)
 
     def handle_response(self, response, url):
-        json_data = json.loads(response)
+        try:
+            json_data = json.loads(response)
+        except InvalidJson as exc:
+            raise InvalidResponseException(exc.message)
+
         if 'url' not in json_data:
             json_data['url'] = url
+        if 'title' not in json_data:
+            json_data['title'] = json_data['url']
+
         return json_data
 
 
@@ -87,6 +94,19 @@ def url_cache(fn):
     return inner
 
 
+def fetch(request):
+    resp = urlopen(request)
+    if resp.code < 200 or resp.code >= 300:
+        return False
+
+    # by RFC, default HTTP charset is ISO-8859-1
+    charset = get_charset(resp) or 'iso-8859-1'
+
+    content = resp.read().decode(charset)
+    resp.close()
+    return content
+
+
 class ProviderRegistry(object):
     def __init__(self, cache=None):
         self._registry = {}
@@ -130,8 +150,8 @@ def bootstrap_basic(cache=None):
     pr.register('https?://(www\.)?dailymotion\.com/\S+', Provider('http://www.dailymotion.com/services/oembed'))
 
     # f
-    pr.register('http://\S*?flickr.com/\S+', Provider('http://www.flickr.com/services/oembed/'))
-    pr.register('http://flic\.kr/\S*', Provider('http://www.flickr.com/services/oembed/'))
+    pr.register('https?://\S*?flickr.com/\S+', Provider('https://www.flickr.com/services/oembed/'))
+    pr.register('https?://flic\.kr/\S*', Provider('https://www.flickr.com/services/oembed/'))
     pr.register('https?://(www\.)?funnyordie\.com/videos/\S+', Provider('http://www.funnyordie.com/oembed'))
 
     # g
@@ -173,7 +193,7 @@ def bootstrap_basic(cache=None):
     pr.register('https?://(www\.)?scribd\.com/\S*', Provider('http://www.scribd.com/services/oembed'))
 
     # t
-    pr.register('https?://(www\.)?twitter.com/\S+/status(es)?/\S+', Provider('http://api.twitter.com/1/statuses/oembed.json'))
+    pr.register('https?://(www\.)?twitter.com/\S+/status(es)?/\S+', Provider('https://api.twitter.com/1/statuses/oembed.json'))
 
     # v
     pr.register('http://\S*.viddler.com/\S*', Provider('http://lab.viddler.com/services/oembed/'))
@@ -181,7 +201,8 @@ def bootstrap_basic(cache=None):
     pr.register('https://vimeo.com/\S+', Provider('https://vimeo.com/api/oembed.json'))
 
     # y
-    pr.register('https?://(\S*.)?youtu(\.be/|be\.com/watch)\S+', Provider('http://www.youtube.com/oembed'))
+    pr.register('http://(\S*.)?youtu(\.be/|be\.com/watch)\S+', Provider('http://www.youtube.com/oembed'))
+    pr.register('https://(\S*.)?youtu(\.be/|be\.com/watch)\S+', Provider('http://www.youtube.com/oembed?scheme=https&'))
     pr.register('http://(\S*\.)?yfrog\.com/\S*', Provider('http://www.yfrog.com/api/oembed'))
 
     # w
@@ -198,10 +219,7 @@ def bootstrap_embedly(cache=None, **params):
     pr = ProviderRegistry(cache)
 
     # fetch the schema
-    resp = urlopen(schema_url)
-    contents = resp.read()
-    resp.close()
-
+    contents = fetch(schema_url)
     json_data = json.loads(contents)
 
     for provider_meta in json_data:
@@ -217,13 +235,25 @@ def bootstrap_noembed(cache=None, **params):
     pr = ProviderRegistry(cache)
 
     # fetch the schema
-    resp = urlopen(schema_url)
-    contents = resp.read()
-    resp.close()
-
+    contents = fetch(schema_url)
     json_data = json.loads(contents)
 
     for provider_meta in json_data:
         for regex in provider_meta['patterns']:
             pr.register(regex, Provider(endpoint, **params))
     return pr
+
+
+def bootstrap_oembedio(cache=None, **params):
+    endpoint = 'http://oembed.io/api'
+    schema_url = 'http://oembed.io/providers'
+
+    pr = ProviderRegistry(cache)
+
+    # fetch the schema
+    contents = fetch(schema_url)
+    json_data = json.loads(contents)
+
+    for provider_meta in json_data:
+        pr.register(provider_meta['s'], Provider(endpoint, **params))
+    return pr
